/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.api.animation.xml;

import org.osgi.framework.ServiceException;
import org.robokind.api.animation.Animation;
import org.robokind.api.common.osgi.ClassTracker;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class AnimationXML {
    public final static String ANIMATION = "Animation";
    public final static String ANIMATION_VERSION_TYPE = "Animation";

    public final static String CHANNELS = "Channels";
    public final static String CHANNEL = "Channel";
    public final static String CHANNEL_ID = "id";
    public final static String CHANNEL_NAME = "name";

    public final static String MOTION_PATHS = "MotionPaths";
    public final static String MOTION_PATH = "MotionPath";
    public final static String MOTION_PATH_NAME = "name";
    public final static String INTERPOLATION_VERSION_TYPE = "Interpolation";

    public final static String CONTROL_POINTS = "ControlPoints";
    public final static String CONTROL_POINT = "ControlPoint";
    public final static String TIME = "Time";
    public final static String POSITION = "Position";
    
    public final static String NAMESPACE = null;

    private static ClassTracker<AnimationFileReader> myReaderTracker;
    private static ClassTracker<AnimationFileWriter> myWriterTracker;

    public static Animation loadAnimation(String path) 
            throws ServiceException, Throwable {
        AnimationFileReader reader = getReader();
        if(reader == null){
            throw new ServiceException("No AnimationFileReader Service found.");
        }
        return reader.readAnimation(path);
    }
    
    public static void saveAnimation(String file, Animation a) 
            throws ServiceException, Throwable{
        AnimationFileWriter writer = getWriter();
        if(writer == null){
            throw new ServiceException("No AnimationFileWriter Service found.");
        }
        writer.writeAnimation(file, a);
    }
    
    private static AnimationFileReader getReader(){
        if(myReaderTracker == null){
            myReaderTracker = ClassTracker.build(AnimationFileReader.class, "");
        }
        if(myReaderTracker == null){
            return null;
        }
        return myReaderTracker.getTopService();
    }
        
    private static AnimationFileWriter getWriter(){
        if(myWriterTracker == null){
            myWriterTracker = ClassTracker.build(AnimationFileWriter.class, "");
        }
        if(myWriterTracker == null){
            return null;
        }
        return myWriterTracker.getTopService();
    }
}
