/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.api.interpolation;

import org.robokind.api.common.utils.Factory;
import org.robokind.api.common.config.VersionProperty;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public abstract class InterpolatorFactory implements Factory<Interpolator> {

    @Override
    public abstract Interpolator create();
    /**
     * Returns the VersionProperty of the Interpolator created by the Factory.
     * @return VersionProperty of the Interpolator created by the Factory
     */
    public abstract VersionProperty getVersion();

    /**
     * Static utility method for registering new InterpolatorFactories with the
     * InterpolatorDirector, making it accessible to the rest of the platform.
     * @param factory InterpolatorFactory to register
     */
    protected static void register(InterpolatorFactory factory){
        InterpolatorDirectory.registerFactory(factory);
    }
}
