/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.integration.netbeans;

import java.awt.Image;
import org.openide.nodes.BeanNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class BundleChildren extends Children.Keys<Bundle> implements BundleListener {
    private BundleContext myContext;

    BundleChildren(BundleContext context){
        myContext = context;
    }

    void setContext(BundleContext context){
        myContext = context;
    }

    public BundleContext getContext(){
        return myContext;
    }


    @Override
    protected void addNotify() {
        myContext.addBundleListener(this);
        bundleChanged(null);
    }

    @Override
    protected void removeNotify() {
        myContext.removeBundleListener(this);
    }

    @Override
    protected Node[] createNodes(Bundle t) {
        try {
            Node bn = new BeanNode<Bundle>(t) {
                @Override
                public Image getIcon(int type) {
                    if (getBean().getLocation().startsWith("netigso:/")) { // NOI18N
                        return ImageUtilities.loadImage("org/robokind/nbui/monitor/module.png");
                    } else {
                        return ImageUtilities.loadImage("org/robokind/nbui/monitor/bundle.png");
                    }
                }

                @Override
                public Image getOpenedIcon(int type) {
                    return getIcon(type);
                }
            };
            bn.setName(t.getSymbolicName());
            return new Node[]{bn};
        } catch (Exception ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    @Override
    public void bundleChanged(BundleEvent be) {
        setKeys(myContext.getBundles());
    }
}