/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.integration.netbeans;

import java.awt.Image;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.nodes.BeanNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class ServiceChildren extends Children.Keys<ServiceReference> implements ServiceListener {
    private final static Logger theLogger = Logger.getLogger(ServiceChildren.class.getName());
    private BundleContext myContext;

    ServiceChildren(BundleContext context){
        myContext = context;
    }

    void setContext(BundleContext context){
        myContext = context;
    }

    public BundleContext getContext(){
        return myContext;
    }


    @Override
    protected void addNotify() {
        myContext.addServiceListener(this);
        serviceChanged(null);
    }

    @Override
    protected void removeNotify() {
        myContext.removeServiceListener(this);
    }

    @Override
    protected Node[] createNodes(ServiceReference t) {
        try {
            Node bn = new BeanNode<ServiceReference>(t) {
                @Override
                public Image getIcon(int type) {
                    //if (getBean().getBundle().getLocation().startsWith("netigso:/")) { // NOI18N
                    //    return ImageUtilities.loadImage("org/robokind/nbui/monitor/module.png");
                    //} else {
                        return ImageUtilities.loadImage("org/robokind/nbui/monitor/bundle.png");
                    //}
                }

                @Override
                public Image getOpenedIcon(int type) {
                    return getIcon(type);
                }
            };
            Object service = myContext.getService(t);
            bn.setName(service.getClass().getSimpleName());
            return new Node[]{bn};
        } catch (Exception ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    @Override
    public void serviceChanged(ServiceEvent event) {
        try{
            setKeys(myContext.getAllServiceReferences(null, null));
        }catch(InvalidSyntaxException ex){
            theLogger.log(Level.WARNING, "There was an error updating the Service list.", ex);
        }
    }

}
