/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.nbui.monitor;

import java.util.Collection;
import java.util.logging.Logger;
import org.openide.explorer.ExplorerManager;
import org.openide.nodes.AbstractNode;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import org.robokind.integration.netbeans.ServiceChildren;

/**
 * 
 * @author Matthew Stevenson <www.robokind.org>
 */
public class ServiceMonitorTopComponent extends TopComponent
implements ExplorerManager.Provider, LookupListener {
    private Lookup.Result result = null;
    private ExplorerManager em;
    private static ServiceMonitorTopComponent instance;
    private static final String PREFERRED_ID = "ServiceMonitorTopComponent";
    private boolean myIsSet;

    public ServiceMonitorTopComponent() {
        myIsSet = false;
        setIcon(ImageUtilities.loadImage("org/robokind/nbui/monitor/bundle.png"));
        this.em = new ExplorerManager();
        initComponents();
        setName(NbBundle.getMessage(ServiceMonitorTopComponent.class, "CTL_ServiceMonitorAction"));
        setToolTipText(NbBundle.getMessage(ServiceMonitorTopComponent.class, "HINT_ServiceMonitorTopComponent"));
    }

    private void setServiceChildren(ServiceChildren bc){
        if(myIsSet){
            return;
        }
        myIsSet = true;
        AbstractNode root = new AbstractNode(bc);
        root.setName("bundles");
        root.setIconBaseWithExtension("org/robokind/nbui/monitor/bundle.png");
        root.setDisplayName(NbBundle.getMessage(ServiceMonitorTopComponent.class, "LAB_AllServices"));
        em.setRootContext(root);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        split = new javax.swing.JSplitPane();
        tree = new org.openide.explorer.view.BeanTreeView();
        sheet = new org.openide.explorer.propertysheet.PropertySheetView();

        split.setLeftComponent(tree);
        split.setRightComponent(sheet);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(split, javax.swing.GroupLayout.DEFAULT_SIZE, 1168, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(split, javax.swing.GroupLayout.DEFAULT_SIZE, 486, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private org.openide.explorer.propertysheet.PropertySheetView sheet;
    private javax.swing.JSplitPane split;
    private org.openide.explorer.view.BeanTreeView tree;
    // End of variables declaration//GEN-END:variables
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link #findInstance}.
     */
    public static synchronized ServiceMonitorTopComponent getDefault() {
        if (instance == null) {
            instance = new ServiceMonitorTopComponent();
        }
        return instance;
    }

    /**
     * Obtain the ShowBundlesTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized ServiceMonitorTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            Logger.getLogger(ServiceMonitorTopComponent.class.getName()).warning(
                    "Cannot find " + PREFERRED_ID + " component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof ServiceMonitorTopComponent) {
            return (ServiceMonitorTopComponent) win;
        }
        Logger.getLogger(ServiceMonitorTopComponent.class.getName()).warning(
                "There seem to be multiple components with the '" + PREFERRED_ID
                + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_NEVER;//PERSISTENCE_ALWAYS;
    }

    @Override
    public void componentOpened() {
        result = Utilities.actionsGlobalContext().lookupResult(ServiceChildren.class);
        ServiceChildren bc = Lookup.getDefault().lookup(ServiceChildren.class);
        if(bc != null){
            setServiceChildren(bc);
        }
        result.addLookupListener(this);
    }

    @Override
    public void componentClosed() {
        result.removeLookupListener (this);
        result = null;
    }

    void writeProperties(java.util.Properties p) {
        // better to version settings since initial version as advocated at
        // http://wiki.apidesign.org/wiki/PropertyFiles
        p.setProperty("version", "1.0");
    }

    Object readProperties(java.util.Properties p) {
        if (instance == null) {
            instance = this;
        }
        instance.readPropertiesImpl(p);
        return instance;
    }

    private void readPropertiesImpl(java.util.Properties p) {
        String version = p.getProperty("version");
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }

    @Override
    public ExplorerManager getExplorerManager() {
        return em;
    }

    @Override
    public void resultChanged(LookupEvent lookupEvent) {
        Lookup.Result r = (Lookup.Result) lookupEvent.getSource();
        Collection c = r.allInstances();
        if (!c.isEmpty()) {
            setServiceChildren((ServiceChildren)c.iterator().next());
        }
    }
}
