/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.nbui.motion.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;
import org.robokind.api.motion.utils.JointGroup;
import org.robokind.impl.motion.utils.RobotJointGroupConfigXMLReader;
import org.robokind.impl.motion.utils.RobotJointGroup;
import org.robokind.api.common.services.ServiceConnectionDirectory;

/**
 * 
 * @author Matthew Stevenson <www.robokind.org>
 */
public final class LoadJointGroup implements ActionListener {
    private final static Logger theLogger = Logger.getLogger(LoadJointGroup.class.getName());

    @Override
    public void actionPerformed(ActionEvent e) {
        JFileChooser chooser = new JFileChooser();
        int ret = chooser.showOpenDialog(null);
        if(ret == JFileChooser.CANCEL_OPTION){
            return;
        }
        File file = chooser.getSelectedFile();
        if(file == null){
            return;
        }
        BundleContext context = FrameworkUtil.getBundle(JointGroup.class).getBundleContext();
        JointGroup group = ServiceConnectionDirectory.buildService(
                context, 
                RobotJointGroup.VERSION, 
                RobotJointGroupConfigXMLReader.VERSION, 
                file, 
                File.class,
                JointGroup.class);
        if(group == null){
            return;
        }
        context.registerService(JointGroup.class.getName(), group, new Properties());
        theLogger.log(Level.INFO, "JointGroup Service Registered.");
    }
}
