/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.swingui.animation.actions;

import java.util.Map;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map.Entry;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import org.robokind.api.animation.editor.AnimationEditor;
import org.robokind.api.animation.editor.actions.EditorAction.AddChild;
import org.robokind.api.animation.factory.ChannelFactory;
import org.robokind.api.animation.utils.AnimationUtils;
import org.robokind.api.animation.utils.ChannelsParameterSource;
import org.robokind.swingui.animation.timeline.AnimationTimelinePanel;
import org.robokind.swingui.common.utils.CoordinateScalar;
import org.robokind.swingui.animation.menus.UIMenuItem;
import static org.robokind.api.common.localization.Localizer.$;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public abstract class UIAction{
    
    public static class Zoom implements ActionListener{
        private CoordinateScalar myScalar;
        private double myAmount;
        private AnimationTimelinePanel myPanel;
        /**
         * Creates a Zoom action for a CoordinateScalar.
         * @param label
         * @param s the CoordinateScalar to zoom
         * @param amount the zoom amount
         * @param c
         */
        public Zoom(CoordinateScalar s, AnimationTimelinePanel c, double amount){
            myScalar = s;
            myAmount = amount;
            myPanel = c;
        }
        @Override
        public void actionPerformed(ActionEvent e) {
            myScalar.setScaleX(myScalar.getScaleX()*myAmount);
            int minAnim = myPanel.getAnimationWidth();
            int minPnl = myPanel.getMinimumSize().width;
            JScrollPane scrollPane = (JScrollPane)myPanel.getParent().getParent();
            int minScrl = scrollPane.getWidth() + 1;
            int size = Math.max(minAnim, minPnl);
            size = Math.max(size, minScrl);
            myPanel.setPreferredSize(new Dimension(size, myPanel.getPreferredSize().height));
            myPanel.revalidate();
            scrollPane.repaint();
        }
    }
    
    public static JMenu AddChannelMenu(AnimationEditor controller){
        JMenu menu = new JMenu($("add.channel"));
        ChannelsParameterSource paramSourcre = AnimationUtils.getChannelsParameterSource();
        if(paramSourcre == null){
            return menu;
        }
        Map<Integer,String> names = paramSourcre.getChannelNames();
        if(names == null){
            return menu;
        }
        for(Entry<Integer,String> e : paramSourcre.getChannelNames().entrySet()){
            String label = e.getValue();
            JMenuItem item = null;
            if(controller.containsLogicalId(e.getKey())){
                item = new JMenuItem(label);
                item.setEnabled(false);
            }else{
                ChannelFactory fact = new ChannelFactory(e.getKey(), e.getValue());
                item = new UIMenuItem(label, new AddChild(controller, fact, true));
            }
            menu.add(item);
        }
        return menu;
    }
}
