/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * AnimationScrollPanel.java
 *
 * Created on Feb 15, 2011, 9:50:19 PM
 */

package org.robokind.swingui.animation.timeline.scroll;

import java.awt.Dimension;
import javax.swing.border.EmptyBorder;
import org.robokind.api.animation.editor.AnimationEditor;
import org.robokind.swingui.common.utils.SettingsRepository;
import org.robokind.swingui.common.utils.CoordinateScalar;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class AnimationScrollPanel extends javax.swing.JPanel {
    private AnimationEditor myController;
    private CoordinateScalar myScalar;
    private TimelineZoomScrollListener myScrollWheelListener;
    private EndlessScrollListener myScrollBarListener;
    
    /** Creates new form AnimationScrollPanel */
    public AnimationScrollPanel() {
        initComponents();
        myScrollBarListener = new EndlessScrollListener(15);
        myScrollWheelListener = new TimelineZoomScrollListener();
        myScrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
    }

    /**
     *
     * @param scalar
     */
    public void init(CoordinateScalar scalar){
        myScalar = scalar;
        myPanel.init(myScalar);
        myScrollBarListener.init(myPanel, myScrollPane);
        myScrollWheelListener.init(myPanel, myScrollPane);
        myPanel.addMouseWheelListener(myScrollWheelListener);
    }

    /**
     *
     * @param controller
     */
    public void setController(AnimationEditor controller){
        myController = controller;
        myPanel.setController(controller);
        resetSize();
    }

    /**
     *
     */
    public void resetSize(){
        int w = getParent().getWidth()+1;
        if(w > 1){
            myPanel.setPreferredSize(new Dimension(w, myPanel.getPreferredSize().height));
        }
        if(myController != null){
            double length = myController.getEnd();
            double width = getWidth()-SettingsRepository.theAnimationWidthOffset;

            double scaleX = 0.2;
            if(width > 0 && length > 0){
                scaleX = width/length;
            }
            myScalar.setScaleX(scaleX);
        }
        myPanel.resetSize();
    }

    /**
     *
     * @return
     */
    public CoordinateScalar getScalar(){
        return myScalar;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        myScrollPane = new javax.swing.JScrollPane();
        myPanel = new org.robokind.swingui.animation.timeline.AnimationTimelinePanel();

        myScrollPane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        myScrollPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

        javax.swing.GroupLayout myPanelLayout = new javax.swing.GroupLayout(myPanel);
        myPanel.setLayout(myPanelLayout);
        myPanelLayout.setHorizontalGroup(
            myPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 110, Short.MAX_VALUE)
        );
        myPanelLayout.setVerticalGroup(
            myPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 110, Short.MAX_VALUE)
        );

        myScrollPane.setViewportView(myPanel);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(myScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 56, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(myScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 57, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private org.robokind.swingui.animation.timeline.AnimationTimelinePanel myPanel;
    private javax.swing.JScrollPane myScrollPane;
    // End of variables declaration//GEN-END:variables

}
