/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.swingui.animation.timeline.scroll;

import java.awt.Dimension;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import javax.swing.JScrollPane;
import org.robokind.swingui.animation.timeline.AnimationTimelinePanel;
import org.robokind.swingui.common.utils.CoordinateScalar;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class TimelineZoomScrollListener implements MouseWheelListener {
    private AnimationTimelinePanel myPanel;
    private JScrollPane myScrollPane;
    private double myScrollRate;

    /**
     *
     * @param panel
     * @param pane
     */
    public void init(AnimationTimelinePanel panel, JScrollPane pane){
        myPanel = panel;
        myScrollPane = pane;
        myScrollRate = 0.95;
    }

    @Override
    public void mouseWheelMoved(MouseWheelEvent e) {
        if(myPanel == null || e.isShiftDown()){
            return;
        }
        CoordinateScalar sc = myPanel.getScalar();
        double scroll = e.getWheelRotation();
        scroll = Math.pow(myScrollRate, scroll);
        scroll *= sc.getScaleX();
        if(scroll >= 10){
            return;
        }
        sc.setScaleX(scroll);
        setSize();
    }

    private void setSize(){
        int minAnim = myPanel.getAnimationWidth();
        int minPnl = myPanel.getMinimumSize().width;
        int minScrl = myScrollPane.getWidth() + 1;
        int size = Math.max(minAnim, minPnl);
        size = Math.max(size, minScrl);
        myPanel.setPreferredSize(new Dimension(size, myPanel.getPreferredSize().height));
        myPanel.revalidate();
        myScrollPane.repaint();
    }
}
