/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * ComponentHeaderPanel.java
 *
 * Created on Apr 27, 2011, 7:25:34 PM
 */

package org.robokind.swingui.common;

import java.awt.event.ActionListener;
import java.beans.Beans;
import javax.swing.Icon;
import org.robokind.swingui.common.utils.SettingsRepository;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class ComponentHeaderPanel extends javax.swing.JPanel {

    /** Creates new form ComponentHeaderPanel */
    public ComponentHeaderPanel() {
        initComponents();
        if(Beans.isDesignTime()){
           return;
        }
        Icon[] icons = SettingsRepository.getDefaultHeaderIcons();
        myIcon.setIcon(icons[0]);
        myCloseButton.setIcon(icons[1]);
        myCloseButton.setRolloverIcon(icons[2]);
        myCloseButton.setSelectedIcon(icons[3]);
        myCloseButton.setRolloverEnabled(true);
    }

    /**
     *
     * @param major
     * @param minor
     */
    public void setTitle(String major, String minor){
        myTitle.setText(major);
        if(minor == null || minor.isEmpty()){
            myTitleMinor.setText("");
        }else{
            myTitleMinor.setText(" - " + minor);
        }

    }

    /**
     *
     * @param label
     */
    public void setLabel(String label){
        if(label == null || label.isEmpty()){
            myHeaderLaber.setText("");
        }else{
            myHeaderLaber.setText("(" + label + ")");
        }
    }

    /**
     *
     * @param status
     */
    public void setStatus(String status){
        if(status == null || status.isEmpty()){
            myHeaderStatus.setText(" ");
        }else{
            myHeaderStatus.setText(status + " ");
        }
    }

    /**
     *
     * @param icon
     */
    public void setIcon(Icon icon){
        myIcon.setIcon(icon);
    }

    /**
     *
     * @param icon
     * @param hover
     * @param select
     */
    public void setCloseIcons(Icon icon, Icon hover, Icon select){
        myCloseButton.setIcon(icon);
        myCloseButton.setRolloverIcon(hover);
        myCloseButton.setSelectedIcon(select);
        myCloseButton.setRolloverEnabled(hover == null && select == null);
    }

    /**
     *
     * @param listener
     */
    public void addCloseActionListener(ActionListener listener){
        myCloseButton.addActionListener(listener);
    }

    /**
     *
     * @param listener
     */
    public void removeCloseActionListener(ActionListener listener){
        myCloseButton.removeActionListener(listener);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        myIcon = new javax.swing.JLabel();
        myCloseButton = new javax.swing.JButton();
        myHeaderTextPanel = new javax.swing.JPanel();
        myTitle = new javax.swing.JLabel();
        myTitleMinor = new javax.swing.JLabel();
        myHeaderStatus = new javax.swing.JLabel();
        myHeaderLaber = new javax.swing.JLabel();

        myIcon.setBorder(javax.swing.BorderFactory.createEmptyBorder(4, 4, 0, 0));
        myIcon.setMaximumSize(new java.awt.Dimension(16, 16));
        myIcon.setMinimumSize(new java.awt.Dimension(16, 16));
        myIcon.setPreferredSize(new java.awt.Dimension(16, 16));
        myIcon.setRequestFocusEnabled(false);

        myCloseButton.setToolTipText("Close");
        myCloseButton.setBorder(null);
        myCloseButton.setBorderPainted(false);
        myCloseButton.setContentAreaFilled(false);
        myCloseButton.setFocusable(false);
        myCloseButton.setHideActionText(true);
        myCloseButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        myCloseButton.setMaximumSize(new java.awt.Dimension(16, 16));
        myCloseButton.setMinimumSize(new java.awt.Dimension(16, 16));
        myCloseButton.setPreferredSize(new java.awt.Dimension(16, 16));

        myHeaderTextPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(3, 0, 0, 0));

        myTitle.setFont(new java.awt.Font("Tahoma", 1, 11));
        myTitle.setText("Title");
        myTitle.setIconTextGap(0);

        myTitleMinor.setText("- Minor");

        myHeaderStatus.setFont(new java.awt.Font("Tahoma", 2, 11));
        myHeaderStatus.setText("Status ");

        myHeaderLaber.setText("(Label)");

        javax.swing.GroupLayout myHeaderTextPanelLayout = new javax.swing.GroupLayout(myHeaderTextPanel);
        myHeaderTextPanel.setLayout(myHeaderTextPanelLayout);
        myHeaderTextPanelLayout.setHorizontalGroup(
            myHeaderTextPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(myHeaderTextPanelLayout.createSequentialGroup()
                .addComponent(myTitle)
                .addGap(2, 2, 2)
                .addComponent(myTitleMinor)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 302, Short.MAX_VALUE)
                .addComponent(myHeaderLaber)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(myHeaderStatus))
        );
        myHeaderTextPanelLayout.setVerticalGroup(
            myHeaderTextPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(myHeaderTextPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(myHeaderStatus)
                .addComponent(myHeaderLaber)
                .addComponent(myTitle)
                .addComponent(myTitleMinor))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(myIcon, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(myHeaderTextPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(myCloseButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(myIcon, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)
            .addComponent(myCloseButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
            .addComponent(myHeaderTextPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton myCloseButton;
    private javax.swing.JLabel myHeaderLaber;
    private javax.swing.JLabel myHeaderStatus;
    private javax.swing.JPanel myHeaderTextPanel;
    private javax.swing.JLabel myIcon;
    private javax.swing.JLabel myTitle;
    private javax.swing.JLabel myTitleMinor;
    // End of variables declaration//GEN-END:variables

}
