/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.swingui.common.utils;

import java.awt.geom.Point2D;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class CoordinateScalar {
    private final static double theScaleMax = 5.0;
    private double myScaleX, myScaleY;
    private double myXOffset, myYOffset;
    private boolean myInvertY;

    /**
     *
     */
    public CoordinateScalar(){
        myScaleX = 1.0;
        myScaleY = 1.0;
        myInvertY = true;
        myYOffset = 0;
        myXOffset = 0;
    }

    /**
     *
     * @param x
     * @param y
     * @param i
     */
    public CoordinateScalar(double x, double y, boolean i){
        myScaleX = x;
        myScaleY = y;
        myInvertY = i;
        myYOffset = 0;
        myXOffset = 0;
    }

    /**
     *
     * @return
     */
    public double getScaleX(){
        return myScaleX;
    }

    /**
     *
     * @param s
     */
    public void setScaleX(double s){
        if(s <= 0){
            return;
        }
        myScaleX = Math.min(s, theScaleMax);
    }

    /**
     *
     * @param val
     */
    public void setXOffset(double val){
        myXOffset = val;
    }

    /**
     *
     * @return
     */
    public double getScaleY(){
        return myScaleY;
    }

    /**
     *
     * @param s
     */
    public void setScaleY(double s){
        if(s <= 0){
            return;
        }
        myScaleY = s;
    }

    /**
     *
     * @param val
     */
    public void setYOffset(double val){
        myYOffset = val;
    }

    /**
     *
     * @param x
     * @return
     */
    public int scaleX(double x){
        return (int)(x*myScaleX + myXOffset);
    }
    /**
     *
     * @param p
     * @return
     */
    public int scaleX(Point2D p){
        return scaleX(p.getX());
    }

    /**
     *
     * @param y
     * @return
     */
    public int scaleY(double y){
        if(myInvertY){
            y = 1.0-y;
        }
        return (int)(y*myScaleY + myYOffset);
    }
    /**
     *
     * @param p
     * @return
     */
    public int scaleY(Point2D p){
        return scaleY(p.getY());
    }

    /**
     *
     * @param x
     * @return
     */
    public double unscaleX(int x){
        return (x - myXOffset)/myScaleX;
    }
    /**
     *
     * @param p
     * @return
     */
    public double unscaleX(Point2D p){
        return unscaleX((int)p.getX());
    }

    /**
     *
     * @param y
     * @return
     */
    public double unscaleY(int y){
        double dY = (y - myYOffset)/myScaleY;
        if(myInvertY){
            dY = 1.0-dY;
        }
        return dY;
    }
    /**
     *
     * @param p
     * @return
     */
    public double unscaleY(Point2D p){
        return unscaleY((int)p.getY());
    }

    /**
     *
     */
    public static interface ScalarSource{
        /**
         *
         * @return
         */
        public CoordinateScalar getScalar();
    }
}
