/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * JointTreeFrame.java
 *
 * Created on Aug 20, 2011, 1:25:55 AM
 */
package org.robokind.swingui.motion.jointgroup;

import java.beans.Beans;
import java.io.File;
import java.util.Properties;
import org.osgi.framework.BundleContext;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.utils.JointGroup;
import org.robokind.api.motion.servos.config.ServoRobotConfig;
import org.robokind.api.motion.servos.utils.ServoRobotConnector;
import org.robokind.impl.motion.config.RobotConfigXMLFileLoader;
import org.robokind.impl.motion.utils.RobotJointGroupXMLFileLoader;
import org.robokind.impl.motion.utils.RobotJointGroup;
import org.robokind.impl.motion.utils.RobotJointGroupFactory;
import org.robokind.api.common.osgi.OSGiUtils;
import org.robokind.api.motion.Robot.Id;
import org.robokind.impl.motion.utils.RobotJointGroupConfig;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JointTreeFrame extends javax.swing.JFrame {

    /** Creates new form JointTreeFrame */
    public JointTreeFrame() {
        initComponents();
        if(Beans.isDesignTime()){
            return;
        }
    }
    
    private void registerGroup(){
        File file = new File("/home/matt/Desktop/jg.xml");
        RobotJointGroupXMLFileLoader loader = 
                new RobotJointGroupXMLFileLoader();
        RobotJointGroupConfig config = loader.loadConfiguration(file);
        Id robotId = config.getRobotId();
        if(robotId == null){
            throw new NullPointerException();
        }
        RobotJointGroup group = 
                (RobotJointGroup)new RobotJointGroupFactory().build(config);
        if(group == null){
            return;
        }
        BundleContext context = OSGiUtils.getBundleContext(JointGroup.class);
        Properties props = new Properties();
        props.put(Robot.PROP_ID, robotId.toString());
        context.registerService(JointGroup.class.getName(), group, props);
    }
    
    private void registerRobot(){
        File file2 = new File("/home/matt/Desktop/dynaConf.xml");
        RobotConfigXMLFileLoader dl = new RobotConfigXMLFileLoader();
        ServoRobotConfig conf = dl.loadConfiguration(file2);
        ServoRobotConnector dc = new ServoRobotConnector();
        Robot dev = dc.build(conf);
        dev.connect();
        BundleContext context = OSGiUtils.getBundleContext(Robot.class);
        if(context == null){
            return;
        }
        context.registerService(Robot.class.getName(), dev, null);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jointTreePanel1 = new org.robokind.swingui.motion.jointgroup.JointTreePanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jButton1.setText("jButton1");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton2.setText("jButton2");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jButton3.setText("jButton3");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jointTreePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 345, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jButton1)
                    .addComponent(jButton2)
                    .addComponent(jButton3)))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jButton1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jButton2)
                .addGap(18, 18, 18)
                .addComponent(jButton3)
                .addGap(192, 192, 192))
            .addComponent(jointTreePanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 312, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        registerGroup();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        registerRobot();
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        jointTreePanel1.readJointProperties();
    }//GEN-LAST:event_jButton3ActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                new JointTreeFrame().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private org.robokind.swingui.motion.jointgroup.JointTreePanel jointTreePanel1;
    // End of variables declaration//GEN-END:variables
}
