/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * JointTreePanel.java
 *
 * Created on Aug 20, 2011, 1:23:31 AM
 */
package org.robokind.swingui.motion.jointgroup;

import java.beans.Beans;
import java.util.Arrays;
import java.util.List;
import org.netbeans.swing.outline.DefaultOutlineModel;
import org.netbeans.swing.outline.OutlineModel;
import org.osgi.framework.BundleContext;
import org.robokind.api.motion.Joint;
import org.robokind.api.motion.utils.JointGroup;
import org.robokind.api.motion.JointProperty;
import org.robokind.api.common.osgi.OSGiUtils;
import org.robokind.api.common.osgi.ServiceClassListener;
import org.robokind.api.motion.joint_properties.ReadLoad;
import org.robokind.api.motion.joint_properties.ReadTemperature;
import org.robokind.api.motion.joint_properties.ReadVoltage;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JointTreePanel extends javax.swing.JPanel {
    private JointGroup myRootGroup;
    private JointTreeRowModel myRowModel;
    private JointGroupTreeModel myTreeModel;
    private JointTreeJointGroupListener myListenerTree;
    private List<String> myPropertyNames;
    private ServiceClassListener<JointGroup> myGroupTracker;
    /** Creates new form JointTreePanel */
    public JointTreePanel() {
        initComponents();
        if(Beans.isDesignTime()){
            return;
        }
        myPropertyNames = Arrays.asList(
                ReadLoad.PROPERTY_NAME, 
                ReadTemperature.PROPERTY_NAME, 
                ReadVoltage.PROPERTY_NAME);
        BundleContext context = OSGiUtils.getBundleContext(JointGroup.class);
        myGroupTracker = new ServiceClassListener<JointGroup>(JointGroup.class, context, null) {

            @Override
            protected void addService(JointGroup t) {
                setJointGroup();
            }

            @Override
            protected void removeService(JointGroup t) {
                setJointGroup();
            }
            
            private void setJointGroup(){
                if(myList.isEmpty()){
                    //setModel(myRootGroup, myPropertyNames);
                    return;
                }
                JointGroup group = myList.get(0);
                if(group == myRootGroup){
                    return;
                }
                setModel(group, myPropertyNames);
            }
        };
    }
    
    protected void setModel(JointGroup rootGroup, List<String> propNames){
        myRootGroup = rootGroup;
        myPropertyNames = propNames;
        myRowModel = new JointTreeRowModel(myPropertyNames);
        myTreeModel = new JointGroupTreeModel(rootGroup);
        OutlineModel mdl = DefaultOutlineModel.createOutlineModel(myTreeModel, myRowModel);
        myJointTreeOutline.setRootVisible(true);
        myJointTreeOutline.setModel(mdl);
        myJointTreeOutline.setRenderDataProvider(new JointTreeRenderer());
        jScrollPane1.setViewportView(myJointTreeOutline);
        if(myListenerTree != null){
            myListenerTree.stopListening();
        }
        myListenerTree = JointTreeJointGroupListener.buildListenerTree(myRootGroup, myJointTreeOutline, myPropertyNames);
        
    }
    
    public void readJointProperties(){
        readJointPropertiesRec(myRootGroup, myPropertyNames);
    }
    
    private void readJointPropertiesRec(JointGroup<?,JointGroup,Joint> group, List<String> props){
        for(Joint j : group.getJoints()){
            if(j == null){
                continue;
            }
            for(String s : props){
                JointProperty jp = j.getProperty(s);
                if(jp == null || !jp.getReadable()){
                    continue;
                }
                jp.getValue();
            }
        }
        for(JointGroup g : group.getJointGroups()){
            readJointPropertiesRec(g, props);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        myJointTreeOutline = new org.netbeans.swing.outline.Outline();

        jScrollPane1.setViewportView(myJointTreeOutline);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 161, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private org.netbeans.swing.outline.Outline myJointTreeOutline;
    // End of variables declaration//GEN-END:variables
}
